"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __name = (target, value) => __defProp(target, "name", { value, configurable: true });
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  // If the importer is in node compatibility mode or this is not an ESM
  // file that has been converted to a CommonJS file using a Babel-
  // compatible transform (i.e. "__esModule" has not been set), then set
  // "default" to the CommonJS "module.exports" for node compatibility.
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var operations_sync_exports = {};
__export(operations_sync_exports, {
  computeCMAC: () => computeCMAC,
  computeMAC: () => computeMAC,
  computeNoncePRK: () => computeNoncePRK,
  computePRK: () => computePRK,
  decryptAES128CCM: () => decryptAES128CCM,
  decryptAES128OFB: () => decryptAES128OFB,
  decryptAES256CBC: () => decryptAES256CBC,
  deriveMEI: () => deriveMEI,
  deriveNetworkKeys: () => deriveNetworkKeys,
  deriveTempKeys: () => deriveTempKeys,
  encryptAES128CCM: () => encryptAES128CCM,
  encryptAES128ECB: () => encryptAES128ECB,
  encryptAES128OFB: () => encryptAES128OFB,
  randomBytes: () => randomBytes
});
module.exports = __toCommonJS(operations_sync_exports);
var import_safe = require("@zwave-js/shared/safe");
var import_node_crypto = __toESM(require("node:crypto"), 1);
var import_shared = require("./shared.js");
const Z128 = new Uint8Array(16).fill(0);
const R128 = import_safe.Bytes.from("00000000000000000000000000000087", "hex");
const constantPRK = new Uint8Array(16).fill(51);
const constantTE = new Uint8Array(15).fill(136);
const constantNK = new Uint8Array(15).fill(85);
const constantNonce = new Uint8Array(16).fill(38);
const constantEI = new Uint8Array(15).fill(136);
function randomBytes(length) {
  return import_node_crypto.default.randomBytes(length);
}
__name(randomBytes, "randomBytes");
function encrypt(algorithm, blockSize, trimToInputLength, input, key, iv) {
  const cipher = import_node_crypto.default.createCipheriv(algorithm, key, iv);
  cipher.setAutoPadding(false);
  const { output: plaintext, paddingLength } = (0, import_shared.zeroPad)(input, blockSize);
  const ret = import_safe.Bytes.concat([cipher.update(plaintext), cipher.final()]);
  if (trimToInputLength && paddingLength > 0) {
    return ret.subarray(0, -paddingLength);
  } else {
    return ret;
  }
}
__name(encrypt, "encrypt");
function decrypt(algorithm, blockSize, trimToInputLength, input, key, iv) {
  const cipher = import_node_crypto.default.createDecipheriv(algorithm, key, iv);
  cipher.setAutoPadding(false);
  const { output: ciphertext, paddingLength } = (0, import_shared.zeroPad)(input, blockSize);
  const ret = import_safe.Bytes.concat([cipher.update(ciphertext), cipher.final()]);
  if (trimToInputLength && paddingLength > 0) {
    return ret.subarray(0, -paddingLength);
  } else {
    return ret;
  }
}
__name(decrypt, "decrypt");
function encryptAES128ECB(plaintext, key) {
  return encrypt("aes-128-ecb", 16, false, plaintext, key, new Uint8Array());
}
__name(encryptAES128ECB, "encryptAES128ECB");
const encryptAES128OFB = encrypt.bind(void 0, "aes-128-ofb", 16, true);
const decryptAES128OFB = decrypt.bind(void 0, "aes-128-ofb", 16, true);
const decryptAES256CBC = decrypt.bind(void 0, "aes-256-cbc", 16, true);
function computeMAC(authData, key, iv = new Uint8Array(16).fill(0)) {
  const ciphertext = encrypt("aes-128-cbc", 16, false, authData, key, iv);
  return ciphertext.subarray(-16, -8);
}
__name(computeMAC, "computeMAC");
function generateAES128CMACSubkeys(key) {
  const L = encryptAES128ECB(Z128, key);
  const k1 = !(L[0] & 128) ? (0, import_shared.leftShift1)(L) : (0, import_shared.xor)((0, import_shared.leftShift1)(L), R128);
  const k2 = !(k1[0] & 128) ? (0, import_shared.leftShift1)(k1) : (0, import_shared.xor)((0, import_shared.leftShift1)(k1), R128);
  return [k1, k2];
}
__name(generateAES128CMACSubkeys, "generateAES128CMACSubkeys");
function computeCMAC(message, key) {
  const blockSize = 16;
  const numBlocks = Math.ceil(message.length / blockSize);
  let lastBlock = message.subarray((numBlocks - 1) * blockSize);
  const lastBlockIsComplete = message.length > 0 && message.length % blockSize === 0;
  if (!lastBlockIsComplete) {
    lastBlock = (0, import_shared.zeroPad)(import_safe.Bytes.concat([lastBlock, import_safe.Bytes.from([128])]), blockSize).output;
  }
  let ret = Z128;
  for (let i = 0; i < numBlocks - 1; i++) {
    ret = (0, import_shared.xor)(ret, message.subarray(i * blockSize, (i + 1) * blockSize));
    ret = encryptAES128ECB(ret, key);
  }
  const [k1, k2] = generateAES128CMACSubkeys(key);
  ret = (0, import_shared.xor)(ret, (0, import_shared.xor)(lastBlockIsComplete ? k1 : k2, lastBlock));
  ret = encryptAES128ECB(ret, key);
  return ret.subarray(0, blockSize);
}
__name(computeCMAC, "computeCMAC");
function computePRK(ecdhSharedSecret, pubKeyA, pubKeyB) {
  const message = import_safe.Bytes.concat([ecdhSharedSecret, pubKeyA, pubKeyB]);
  return computeCMAC(message, constantPRK);
}
__name(computePRK, "computePRK");
function deriveTempKeys(PRK) {
  const T1 = computeCMAC(import_safe.Bytes.concat([constantTE, [1]]), PRK);
  const T2 = computeCMAC(import_safe.Bytes.concat([T1, constantTE, [2]]), PRK);
  const T3 = computeCMAC(import_safe.Bytes.concat([T2, constantTE, [3]]), PRK);
  return {
    tempKeyCCM: T1,
    tempPersonalizationString: import_safe.Bytes.concat([T2, T3])
  };
}
__name(deriveTempKeys, "deriveTempKeys");
function deriveNetworkKeys(PNK) {
  const T1 = computeCMAC(import_safe.Bytes.concat([constantNK, [1]]), PNK);
  const T2 = computeCMAC(import_safe.Bytes.concat([T1, constantNK, [2]]), PNK);
  const T3 = computeCMAC(import_safe.Bytes.concat([T2, constantNK, [3]]), PNK);
  const T4 = computeCMAC(import_safe.Bytes.concat([T3, constantNK, [4]]), PNK);
  return {
    keyCCM: T1,
    keyMPAN: T4,
    personalizationString: import_safe.Bytes.concat([T2, T3])
  };
}
__name(deriveNetworkKeys, "deriveNetworkKeys");
function computeNoncePRK(senderEI, receiverEI) {
  const message = import_safe.Bytes.concat([senderEI, receiverEI]);
  return computeCMAC(message, constantNonce);
}
__name(computeNoncePRK, "computeNoncePRK");
function deriveMEI(noncePRK) {
  const T1 = computeCMAC(import_safe.Bytes.concat([
    constantEI,
    [0],
    constantEI,
    [1]
  ]), noncePRK);
  const T2 = computeCMAC(import_safe.Bytes.concat([T1, constantEI, [2]]), noncePRK);
  return import_safe.Bytes.concat([T1, T2]);
}
__name(deriveMEI, "deriveMEI");
function encryptAES128CCM(key, iv, plaintext, additionalData, authTagLength) {
  const algorithm = `aes-128-ccm`;
  const cipher = import_node_crypto.default.createCipheriv(algorithm, key, iv, { authTagLength });
  cipher.setAAD(additionalData, { plaintextLength: plaintext.length });
  const ciphertext = cipher.update(plaintext);
  cipher.final();
  const authTag = cipher.getAuthTag();
  return { ciphertext, authTag };
}
__name(encryptAES128CCM, "encryptAES128CCM");
function decryptAES128CCM(key, iv, ciphertext, additionalData, authTag) {
  const algorithm = `aes-128-ccm`;
  const decipher = import_node_crypto.default.createDecipheriv(algorithm, key, iv, {
    authTagLength: authTag.length
  });
  decipher.setAuthTag(authTag);
  decipher.setAAD(additionalData, { plaintextLength: ciphertext.length });
  const plaintext = decipher.update(ciphertext);
  let authOK = false;
  try {
    decipher.final();
    authOK = true;
  } catch {
  }
  return { plaintext, authOK };
}
__name(decryptAES128CCM, "decryptAES128CCM");
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  computeCMAC,
  computeMAC,
  computeNoncePRK,
  computePRK,
  decryptAES128CCM,
  decryptAES128OFB,
  decryptAES256CBC,
  deriveMEI,
  deriveNetworkKeys,
  deriveTempKeys,
  encryptAES128CCM,
  encryptAES128ECB,
  encryptAES128OFB,
  randomBytes
});
//# sourceMappingURL=operations.sync.js.map
