"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __name = (target, value) => __defProp(target, "name", { value, configurable: true });
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  // If the importer is in node compatibility mode or this is not an ESM
  // file that has been converted to a CommonJS file using a Babel-
  // compatible transform (i.e. "__esModule" has not been set), then set
  // "default" to the CommonJS "module.exports" for node compatibility.
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var shared_exports = {};
__export(shared_exports, {
  ZWaveLogContainer: () => ZWaveLogContainer,
  ZWaveLoggerBase: () => ZWaveLoggerBase,
  createDefaultTransportFormat: () => createDefaultTransportFormat,
  createLogMessagePrinter: () => createLogMessagePrinter,
  createLoggerFormat: () => createLoggerFormat,
  logMessageFormatter: () => logMessageFormatter,
  messageFitsIntoOneLine: () => messageFitsIntoOneLine,
  messageRecordToLines: () => messageRecordToLines,
  messageToLines: () => messageToLines,
  restoreSilence: () => restoreSilence,
  tagify: () => tagify,
  unsilence: () => unsilence
});
module.exports = __toCommonJS(shared_exports);
var import_shared = require("@zwave-js/shared");
var path = __toESM(require("node:path"), 1);
var import_triple_beam = require("triple-beam");
var import_winston = __toESM(require("winston"), 1);
var import_winston_daily_rotate_file = __toESM(require("winston-daily-rotate-file"), 1);
var import_Colorizer = require("./Colorizer.js");
var import_shared_safe = require("./shared_safe.js");
const { combine, timestamp, label } = import_winston.default.format;
const loglevels = import_triple_beam.configs.npm.levels;
const isTTY = process.stdout.isTTY;
const isUnitTest = process.env.NODE_ENV === "test";
class ZWaveLoggerBase {
  static {
    __name(this, "ZWaveLoggerBase");
  }
  constructor(loggers, logLabel) {
    this.container = loggers;
    this.logger = this.container.getLogger(logLabel);
  }
  logger;
  container;
}
class ZWaveLogContainer extends import_winston.default.Container {
  static {
    __name(this, "ZWaveLogContainer");
  }
  fileTransport;
  consoleTransport;
  loglevelVisibleCache = /* @__PURE__ */ new Map();
  logConfig = {
    enabled: true,
    level: getTransportLoglevel(),
    logToFile: !!process.env.LOGTOFILE,
    maxFiles: 7,
    nodeFilter: (0, import_shared_safe.stringToNodeList)(process.env.LOG_NODES),
    transports: void 0,
    filename: path.join(process.cwd(), `zwavejs_%DATE%.log`),
    forceConsole: false
  };
  constructor(config = {}) {
    super();
    this.updateConfiguration(config);
  }
  getLogger(label2) {
    if (!this.has(label2)) {
      this.add(label2, {
        transports: this.getAllTransports(),
        format: createLoggerFormat(label2),
        // Accept all logs, no matter what. The individual loggers take care
        // of filtering the wrong loglevels
        level: "silly"
      });
    }
    return this.get(label2);
  }
  updateConfiguration(config) {
    for (const key of import_shared_safe.nonUndefinedLogConfigKeys) {
      if (key in config && config[key] === void 0) {
        delete config[key];
      }
    }
    const changedLoggingTarget = config.logToFile != void 0 && config.logToFile !== this.logConfig.logToFile || config.forceConsole != void 0 && config.forceConsole !== this.logConfig.forceConsole;
    if (typeof config.level === "number") {
      config.level = loglevelFromNumber(config.level);
    }
    const changedLogLevel = config.level != void 0 && config.level !== this.logConfig.level;
    if (config.filename != void 0 && !config.filename.includes("%DATE%")) {
      config.filename += "_%DATE%.log";
    }
    const changedFilename = config.filename != void 0 && config.filename !== this.logConfig.filename;
    if (config.maxFiles != void 0) {
      if (typeof config.maxFiles !== "number" || config.maxFiles < 1 || config.maxFiles > 365) {
        delete config.maxFiles;
      }
    }
    const changedMaxFiles = config.maxFiles != void 0 && config.maxFiles !== this.logConfig.maxFiles;
    this.logConfig = Object.assign(this.logConfig, config);
    if (changedLogLevel) {
      this.loglevelVisibleCache.clear();
    }
    const recreateInternalTransports = this.fileTransport == void 0 && this.consoleTransport == void 0 || changedLoggingTarget || changedFilename || changedMaxFiles;
    if (recreateInternalTransports) {
      this.fileTransport?.destroy();
      this.fileTransport = void 0;
      this.consoleTransport?.destroy();
      this.consoleTransport = void 0;
    }
    if (recreateInternalTransports || config.transports != void 0) {
      this.loggers.forEach((logger) => logger.configure({ transports: this.getAllTransports() }));
    }
  }
  getConfiguration() {
    return this.logConfig;
  }
  /** Tests whether a log using the given loglevel will be logged */
  isLoglevelVisible(loglevel) {
    if (!this.fileTransport && !this.consoleTransport && (!this.logConfig.transports || this.logConfig.transports.length === 0)) {
      return false;
    }
    if (!this.loglevelVisibleCache.has(loglevel)) {
      this.loglevelVisibleCache.set(loglevel, loglevel in loglevels && loglevels[loglevel] <= loglevels[this.logConfig.level]);
    }
    return this.loglevelVisibleCache.get(loglevel);
  }
  destroy() {
    for (const key in this.loggers) {
      this.close(key);
    }
    this.fileTransport = void 0;
    this.consoleTransport = void 0;
    this.logConfig.transports = [];
  }
  getAllTransports() {
    return [
      ...this.getInternalTransports(),
      ...this.logConfig.transports ?? []
    ];
  }
  getInternalTransports() {
    const ret = [];
    if (!this.logConfig.enabled) {
      return ret;
    }
    if (this.logConfig.logToFile) {
      if (!this.fileTransport) {
        this.fileTransport = this.createFileTransport();
      }
      ret.push(this.fileTransport);
    }
    if (
      // when in production
      !isUnitTest && (isTTY && !this.logConfig.logToFile || this.logConfig.forceConsole)
    ) {
      if (!this.consoleTransport) {
        this.consoleTransport = this.createConsoleTransport();
      }
      ret.push(this.consoleTransport);
    }
    return ret;
  }
  createConsoleTransport() {
    return new import_winston.default.transports.Console({
      format: createDefaultTransportFormat(
        // Only colorize the output if logging to a TTY, otherwise we'll get
        // ansi color codes in logfiles or redirected shells
        isTTY || isUnitTest,
        // Only use short timestamps if logging to a TTY
        isTTY
      ),
      silent: this.isConsoleTransportSilent()
    });
  }
  isConsoleTransportSilent() {
    return process.env.NODE_ENV === "test" || !this.logConfig.enabled;
  }
  isFileTransportSilent() {
    return !this.logConfig.enabled;
  }
  createFileTransport() {
    const ret = new import_winston_daily_rotate_file.default({
      filename: this.logConfig.filename,
      auditFile: `${this.logConfig.filename.replace("_%DATE%", "_logrotate").replace(/\.log$/, "")}.json`,
      datePattern: "YYYY-MM-DD",
      createSymlink: true,
      symlinkName: path.basename(this.logConfig.filename).replace(`_%DATE%`, "_current"),
      zippedArchive: true,
      maxFiles: `${this.logConfig.maxFiles}d`,
      format: createDefaultTransportFormat(false, false),
      silent: this.isFileTransportSilent()
    });
    ret.on("new", (newFilename) => {
      console.log(`Logging to file:
	${newFilename}`);
    });
    ret.on("error", (err) => {
      console.error(`Error in file stream rotator: ${err.message}`);
    });
    return ret;
  }
  /**
   * Checks the log configuration whether logs should be written for a given node id
   */
  shouldLogNode(nodeId) {
    if (!this.logConfig.nodeFilter)
      return true;
    return this.logConfig.nodeFilter.includes(nodeId);
  }
}
function getTransportLoglevel() {
  return process.env.LOGLEVEL in loglevels ? process.env.LOGLEVEL : "debug";
}
__name(getTransportLoglevel, "getTransportLoglevel");
function loglevelFromNumber(numLevel) {
  if (numLevel == void 0)
    return;
  for (const [level, value] of Object.entries(loglevels)) {
    if (value === numLevel)
      return level;
  }
}
__name(loglevelFromNumber, "loglevelFromNumber");
function createLoggerFormat(channel) {
  return combine(
    // add the channel as a label
    label({ label: channel }),
    // default to short timestamps
    timestamp()
  );
}
__name(createLoggerFormat, "createLoggerFormat");
function createLogMessagePrinter(shortTimestamps) {
  return {
    transform: /* @__PURE__ */ __name((info) => {
      const messageLines = messageToLines(info.message);
      if (info.secondaryTagPadding == void 0) {
        info.secondaryTagPadding = -1;
      }
      let firstLine = [
        info.primaryTags,
        messageLines[0],
        info.secondaryTagPadding < 0 ? void 0 : " ".repeat(info.secondaryTagPadding),
        // If the secondary tag padding is zero, the previous segment gets
        // filtered out and we have one less space than necessary
        info.secondaryTagPadding === 0 && info.secondaryTags ? " " + info.secondaryTags : info.secondaryTags
      ].filter((item) => !!item).join(" ");
      firstLine = `${info.timestamp} ${info.label} ${info.direction}${firstLine}`;
      const lines = [firstLine];
      if (info.multiline) {
        lines.push(...messageLines.slice(1).map((line) => (
          // Skip the columns for the timestamp and the channel name
          (shortTimestamps ? import_shared_safe.timestampPaddingShort : import_shared_safe.timestampPadding) + import_shared_safe.channelPadding + import_shared_safe.directionPrefixPadding + line
        )));
      }
      info[import_triple_beam.MESSAGE] = lines.join("\n");
      return info;
    }, "transform")
  };
}
__name(createLogMessagePrinter, "createLogMessagePrinter");
const logMessageFormatter = {
  transform: /* @__PURE__ */ __name((info) => {
    const messageLines = messageToLines(info.message);
    const firstMessageLineLength = messageLines[0].length;
    info.multiline = messageLines.length > 1 || !messageFitsIntoOneLine(info, info.message.length);
    if (info.secondaryTags) {
      info.secondaryTagPadding = Math.max(
        // -1 has the special meaning that we don't print any padding,
        // because the message takes all the available space
        -1,
        import_shared_safe.LOG_WIDTH - 1 - calculateFirstLineLength(info, firstMessageLineLength)
      );
    }
    if (info.multiline) {
      const lines = [];
      let isFirstLine = true;
      for (let message of messageLines) {
        while (message.length) {
          const cut = Math.min(message.length, isFirstLine ? import_shared_safe.LOG_WIDTH - calculateFirstLineLength(info, 0) - 1 : import_shared_safe.LOG_WIDTH - import_shared_safe.CONTROL_CHAR_WIDTH);
          isFirstLine = false;
          lines.push(message.slice(0, cut));
          message = message.slice(cut);
        }
      }
      info.message = lines.join("\n");
    }
    return info;
  }, "transform")
};
function createDefaultTransportFormat(colorize, shortTimestamps) {
  const formats = [
    // overwrite the default timestamp format if necessary
    shortTimestamps ? timestamp({ format: import_shared_safe.timestampFormatShort }) : void 0,
    logMessageFormatter,
    colorize ? (0, import_Colorizer.colorizer)() : void 0,
    createLogMessagePrinter(shortTimestamps)
  ].filter((f) => !!f);
  return combine(...formats);
}
__name(createDefaultTransportFormat, "createDefaultTransportFormat");
function calculateFirstLineLength(info, firstMessageLineLength) {
  return [
    import_shared_safe.CONTROL_CHAR_WIDTH - 1,
    firstMessageLineLength,
    (info.primaryTags || "").length,
    (info.secondaryTags || "").length
  ].filter((len) => len > 0).reduce((prev, val) => prev + (prev > 0 ? 1 : 0) + val);
}
__name(calculateFirstLineLength, "calculateFirstLineLength");
function messageFitsIntoOneLine(info, messageLength) {
  const totalLength = calculateFirstLineLength(info, messageLength);
  return totalLength <= import_shared_safe.LOG_WIDTH;
}
__name(messageFitsIntoOneLine, "messageFitsIntoOneLine");
function messageToLines(message) {
  if (typeof message === "string") {
    return message.split("\n");
  } else if (message.length > 0) {
    return message;
  } else {
    return [""];
  }
}
__name(messageToLines, "messageToLines");
function messageRecordToLines(message) {
  const entries = Object.entries(message);
  if (!entries.length)
    return [];
  const maxKeyLength = Math.max(...entries.map(([key]) => key.length));
  return (0, import_shared.flatMap)(entries, ([key, value]) => `${key}:${" ".repeat(Math.max(maxKeyLength - key.length + 1, 1))}${value}`.split("\n").map((line) => line.trimEnd()));
}
__name(messageRecordToLines, "messageRecordToLines");
function tagify(tags) {
  return tags.map((pfx) => `[${pfx}]`).join(" ");
}
__name(tagify, "tagify");
function unsilence(logger) {
  const consoleTransport = logger.transports.find((t) => t.name === "console");
  if (consoleTransport) {
    const ret = !!consoleTransport.silent;
    consoleTransport.silent = false;
    return ret;
  }
  return false;
}
__name(unsilence, "unsilence");
function restoreSilence(logger, original) {
  const consoleTransport = logger.transports.find((t) => t.name === "console");
  if (consoleTransport) {
    consoleTransport.silent = original;
  }
}
__name(restoreSilence, "restoreSilence");
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  ZWaveLogContainer,
  ZWaveLoggerBase,
  createDefaultTransportFormat,
  createLogMessagePrinter,
  createLoggerFormat,
  logMessageFormatter,
  messageFitsIntoOneLine,
  messageRecordToLines,
  messageToLines,
  restoreSilence,
  tagify,
  unsilence
});
//# sourceMappingURL=shared.js.map
