"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __name = (target, value) => __defProp(target, "name", { value, configurable: true });
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  // If the importer is in node compatibility mode or this is not an ESM
  // file that has been converted to a CommonJS file using a Babel-
  // compatible transform (i.e. "__esModule" has not been set), then set
  // "default" to the CommonJS "module.exports" for node compatibility.
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var parse_node_exports = {};
__export(parse_node_exports, {
  parseQRCodeString: () => parseQRCodeString,
  parseQRCodeStringAsync: () => parseQRCodeStringAsync
});
module.exports = __toCommonJS(parse_node_exports);
var import_safe = require("@zwave-js/shared/safe");
var import_node_crypto = require("node:crypto");
var import_SecurityClass = require("../definitions/SecurityClass.js");
var import_dsk = require("../dsk/index.js");
var import_Primitive = require("../values/Primitive.js");
var import_definitions = require("./definitions.js");
var import_utils = require("./utils.js");
function parseQRCodeString(qr) {
  qr = qr.trim();
  if (!qr.startsWith("90"))
    (0, import_utils.fail)("must start with 90");
  if (qr.length < import_definitions.minQRCodeLength)
    (0, import_utils.fail)("too short");
  if (!import_definitions.onlyDigitsRegex.test(qr))
    (0, import_utils.fail)("contains invalid characters");
  const version = (0, import_utils.readLevel)(qr, 2);
  if (version > import_definitions.QRCodeVersion.SmartStart)
    (0, import_utils.fail)("invalid version");
  const checksum = (0, import_utils.readUInt16)(qr, 4);
  const hash = (0, import_node_crypto.createHash)("sha1");
  hash.update(import_safe.Bytes.from(qr.slice(9), "ascii"));
  const expectedChecksum = hash.digest().readUInt16BE(0);
  if (checksum !== expectedChecksum)
    (0, import_utils.fail)("invalid checksum");
  const requestedKeysBitmask = (0, import_utils.readUInt8)(qr, 9);
  const requestedSecurityClasses = (0, import_Primitive.parseBitMask)([requestedKeysBitmask], import_SecurityClass.SecurityClass.S2_Unauthenticated);
  if (!requestedSecurityClasses.every((k) => k in import_SecurityClass.SecurityClass)) {
    (0, import_utils.fail)("invalid security class requested");
  }
  let offset = 12;
  const dsk = new import_safe.Bytes(16);
  for (let dskBlock = 0; dskBlock < 8; dskBlock++) {
    const block = (0, import_utils.readUInt16)(qr, offset);
    dsk.writeUInt16BE(block, dskBlock * 2);
    offset += 5;
  }
  const ret = {
    version,
    // This seems like a duplication, but it's more convenient for applications to not have to copy this field over
    requestedSecurityClasses,
    securityClasses: [...requestedSecurityClasses],
    dsk: (0, import_dsk.dskToString)(dsk)
  };
  let hasProductID = false;
  let hasProductType = false;
  while (offset < qr.length) {
    const { entry: { type, ...data }, charsRead } = (0, import_utils.parseTLV)(qr.slice(offset));
    offset += charsRead;
    if (type === import_definitions.ProvisioningInformationType.ProductId) {
      hasProductID = true;
    } else if (type === import_definitions.ProvisioningInformationType.ProductType) {
      hasProductType = true;
    }
    Object.assign(ret, data);
  }
  if (!hasProductID || !hasProductType) {
    (0, import_utils.fail)("missing required fields");
  }
  return ret;
}
__name(parseQRCodeString, "parseQRCodeString");
async function parseQRCodeStringAsync(qr) {
  qr = qr.trim();
  if (!qr.startsWith("90"))
    (0, import_utils.fail)("must start with 90");
  if (qr.length < import_definitions.minQRCodeLength)
    (0, import_utils.fail)("too short");
  if (!import_definitions.onlyDigitsRegex.test(qr))
    (0, import_utils.fail)("contains invalid characters");
  const version = (0, import_utils.readLevel)(qr, 2);
  if (version > import_definitions.QRCodeVersion.SmartStart)
    (0, import_utils.fail)("invalid version");
  const checksum = (0, import_utils.readUInt16)(qr, 4);
  const checksumInput = new TextEncoder().encode(qr.slice(9));
  const subtleCrypto = (
    // FIXME: This method is a copy of the one in parse.browser.ts, except for the node:crypto import
    // Once the following @ts-expect-error is no longer needed, this method should be removed in favor of the one from parse.browser.ts
    // and the exports in index.ts can be updated to be based on index.browser.ts
    //
    // @ts-expect-error Node.js 18 does not support globalThis.crypto yet
    (globalThis.crypto ?? (await import("node:crypto")).webcrypto).subtle
  );
  const hashResult = await subtleCrypto.digest("SHA-1", checksumInput);
  const expectedChecksum = import_safe.Bytes.view(hashResult).readUInt16BE(0);
  if (checksum !== expectedChecksum)
    (0, import_utils.fail)("invalid checksum");
  const requestedKeysBitmask = (0, import_utils.readUInt8)(qr, 9);
  const requestedSecurityClasses = (0, import_Primitive.parseBitMask)([requestedKeysBitmask], import_SecurityClass.SecurityClass.S2_Unauthenticated);
  if (!requestedSecurityClasses.every((k) => k in import_SecurityClass.SecurityClass)) {
    (0, import_utils.fail)("invalid security class requested");
  }
  let offset = 12;
  const dsk = new import_safe.Bytes(16);
  for (let dskBlock = 0; dskBlock < 8; dskBlock++) {
    const block = (0, import_utils.readUInt16)(qr, offset);
    dsk.writeUInt16BE(block, dskBlock * 2);
    offset += 5;
  }
  const ret = {
    version,
    // This seems like a duplication, but it's more convenient for applications to not have to copy this field over
    requestedSecurityClasses,
    securityClasses: [...requestedSecurityClasses],
    dsk: (0, import_dsk.dskToString)(dsk)
  };
  let hasProductID = false;
  let hasProductType = false;
  while (offset < qr.length) {
    const { entry: { type, ...data }, charsRead } = (0, import_utils.parseTLV)(qr.slice(offset));
    offset += charsRead;
    if (type === import_definitions.ProvisioningInformationType.ProductId) {
      hasProductID = true;
    } else if (type === import_definitions.ProvisioningInformationType.ProductType) {
      hasProductType = true;
    }
    Object.assign(ret, data);
  }
  if (!hasProductID || !hasProductType) {
    (0, import_utils.fail)("missing required fields");
  }
  return ret;
}
__name(parseQRCodeStringAsync, "parseQRCodeStringAsync");
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  parseQRCodeString,
  parseQRCodeStringAsync
});
//# sourceMappingURL=parse.node.js.map
